@@ -1,9 +1,29 @@
 AROC.sp <-
-function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pauc = pauccontrol(), p = seq(0,1,l = 101), B = 1000) {
+function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pauc = pauccontrol(), p = seq(0,1,l = 101), B = 1000, parallel = c("no", "multicore", "snow"), ncpus = 1, cl = NULL) {
+    doBoostROC <- function(i, formula.h, data.h, data.d, aroc, est.cdf.h, pauc, p) {    
+        data.boot.d <- data.d[sample(nrow(data.d), replace=TRUE),]
+        data.boot.h <- data.h
+        res.h.b <- sample(aroc$fit$residuals, replace = TRUE)
+        data.boot.h[,marker] <- aroc$fit$fitted + res.h.b
+
+        obj.boot <- compute.AROC(formula.h, data.boot.h, data.boot.d, est.cdf.h, pauc, p)
+        
+        res <- list()
+        res$ROC <- obj.boot$ROC
+        res$AUC <- obj.boot$AUC
+        res$coeff <- coefficients(obj.boot$fit)
+        if(pauc$compute){
+            res$pAUC <- obj.boot$pAUC
+        }
+        res
+    }    
+
     compute.AROC <- function(formula.h, data.h, data.d, est.cdf.h, pauc, p = seq(0,1,l = 101)) {
         np <- length(p)
         
-        marker <- all.vars(formula.h)[1]
+        #marker <- all.vars(formula.h)[1]
+        tf <- terms.formula(formula.h)
+        marker <- as.character(attr(tf, "variables")[2])
         
         # Fit the model in the healthy population
         fit0p <- lm(formula = formula.h, data = data.h)
@@ -23,8 +43,6 @@ function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pa
         aarocp <- 1 - mean(u1)
         if(pauc$compute){
             if(pauc$focus == "FPF"){
-                pu <- seq(0, pauc$value, len = np)
-                arocp_pauc <- apply(outer(u1, pu, "<="), 2, mean)
                 aarocp_pauc <- pauc$value - mean(pmin(pauc$value, u1))
             } else{
                 arocp[1] <- 0
@@ -46,7 +64,11 @@ function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pa
         res$fit <- fit0p
         res
     }
+    pauc <- do.call("pauccontrol", pauc)
+    
     est.cdf.h <- match.arg(est.cdf.h)
+    parallel <- match.arg(parallel)    
+
     np <- length(p)
     if(inherits(formula.h, "character")) {
         formula.h <- as.formula(formula.h)
@@ -60,15 +82,13 @@ function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pa
     }
     
     # Variables in the model
-    names.cov <- all.vars(formula.h)[-1]
+    names.cov <- get_vars_formula(formula.h) #all.vars(formula.h)[-1]
     
     if(sum(is.na(match(c(marker, names.cov, group), names(data)))))
     stop("Not all needed variables are supplied in data")
     if(length(unique(data[,group])) != 2)
     stop(paste(group," variable must have only two different values (for healthy and diseased individuals)"), sep = "")
     
-    pauc <- do.call("pauccontrol", pauc)
-    
     # New data, removing missing values
     data.new <- data[,c(marker,group,names.cov)]
     omit.h <- apply(data.new[data.new[,group] == tag.h, c(marker, group, names.cov)], 1, anyNA)
@@ -87,29 +107,53 @@ function(formula.h, group, tag.h, data, est.cdf.h = c("normal", "empirical"), pa
         paarocp <- res.fit$pAUC
     }
     if(B > 0) {
-        # Confidence intervals
-        arocpb <- matrix(0, nrow = np, ncol = B)
-        aarocpb <- numeric(B)
-        coeffpb <- matrix(0, nrow = length(coeffp), ncol = B)
-        if(pauc$compute){
-            paarocpb <- numeric(B)
-        }
-        
-        for(l in 1:B) {
-            # Another option: healthy (residuals) - diseased (original sample)
-            data.boot.d <- data.new.d[sample(nrow(data.new.d), replace=TRUE),]
-            data.boot.h <- data.new.h
-            res.h.b <- sample(res.fit$fit$residuals, replace = TRUE)
-            data.boot.h[,marker] <- res.fit$fit$fitted + res.h.b
-            
-            res.boot <- compute.AROC(formula.h = formula.h, data.h = data.boot.h, data.d = data.boot.d, est.cdf.h = est.cdf.h, pauc = pauc, p = p)
-            arocpb[,l] <- res.boot$ROC
-            aarocpb[l] <- res.boot$AUC
-            coeffpb[,l] <-  coefficients(res.boot$fit)
-
-            if(pauc$compute){
-                paarocpb[l] <- res.boot$pAUC
+        do_mc <- do_snow <- FALSE
+        if (parallel != "no" && ncpus > 1L) {
+            if (parallel == "multicore") {
+                do_mc <- .Platform$OS.type != "windows"
+            } else if (parallel == "snow") {
+                do_snow <- TRUE
             }
+            if (!do_mc && !do_snow) {
+                ncpus <- 1L
+            }       
+            loadNamespace("parallel") # get this out of the way before recording seed
+        }
+        # Seed
+        #if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
+        #seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
+
+        # Apply function
+        resBoot <- if (ncpus > 1L && (do_mc || do_snow)) {
+                if (do_mc) {
+                    parallel::mclapply(seq_len(B), doBoostROC, formula.h = formula.h, data.h = data.new.h, data.d = data.new.d, aroc = res.fit, est.cdf.h = est.cdf.h, pauc = pauc, p = p, mc.cores = ncpus)
+                } else if (do_snow) {                
+                    if (is.null(cl)) {
+                        cl <- parallel::makePSOCKcluster(rep("localhost", ncpus))
+                        if(RNGkind()[1L] == "L'Ecuyer-CMRG") {
+                            parallel::clusterSetRNGStream(cl)
+                        }
+                        res <- parallel::parLapply(cl, seq_len(B), doBoostROC, formula.h = formula.h, data.h = data.new.h, data.d = data.new.d, aroc = res.fit, est.cdf.h = est.cdf.h, pauc = pauc, p = p)
+                        parallel::stopCluster(cl)
+                        res
+                    } else {
+                        if(!inherits(cl, "cluster")) {
+                            stop("Class of object 'cl' is not correct")
+                        } else {
+                            parallel::parLapply(cl, seq_len(B), doBoostROC, formula.h = formula.h, data.h = data.new.h, data.d = data.new.d, aroc = res.fit, est.cdf.h = est.cdf.h, pauc = pauc, p = p)
+                        }                         
+                    }
+                }
+            } else {
+                lapply(seq_len(B), doBoostROC, formula.h = formula.h, data.h = data.new.h, data.d = data.new.d, aroc = res.fit, est.cdf.h = est.cdf.h, pauc = pauc, p = p)
+            }
+
+        resBoot <- simplify2array(resBoot)    
+        arocpb <- simplify2array(resBoot["ROC",])
+        aarocpb <- unlist(resBoot["AUC",])
+        coeffpb <- simplify2array(resBoot["coeff",])
+        if(pauc$compute){
+            paarocpb <- unlist(resBoot["pAUC",])
         }
     }
     columns <-switch(as.character(B>0),"TRUE" = 1:3,"FALSE"=1)
