@@ -1,5 +1,5 @@
 compute.threshold.AROC <-
-function(object, criterion = c("FPF", "YI"), FPF, newdata) {
+function(object, criterion = c("FPF", "YI"), FPF, newdata, parallel = c("no", "multicore", "snow"), ncpus = 1, cl = NULL) {
 	object.class <- class(object)[1]
 	if (!(object.class %in% c("AROC.bnp", "AROC.kernel", "AROC.sp"))) {
 		stop(paste0("This function can not be used for this object class: ", class(object)))
@@ -11,9 +11,9 @@ function(object, criterion = c("FPF", "YI"), FPF, newdata) {
 	}
 	method <- paste0("compute.threshold.", criterion, ".", object.class)
 	if(criterion == "YI") {
-		res <- eval(parse(text = method))(object = object, newdata = newdata)
+		res <- eval(parse(text = method))(object = object, newdata = newdata, parallel = parallel, ncpus = ncpus, cl = cl)
 	} else {
-		res <- eval(parse(text = method))(object = object, FPF = FPF, newdata = newdata)
+		res <- eval(parse(text = method))(object = object, FPF = FPF, newdata = newdata, parallel = parallel, ncpus = ncpus, cl = cl)
 	}
 	res$call <- match.call()
 	res
